#=====================================================================
# Ez a függvény egy sites×species abundancia mátrixból készít
# sites×trait fajszám, sites×trait relatív fajszám,
# sites×trait abundancia, avagy sites×trait relatív abundancia mátrixot.
#
# x                   A sites×species adatok táblázata.
#                     Ha csak prezencia-abszencia adatokat tartalmaz,
#                     akkor a method="abund" a method="specnumb"-al egyező
#                     eredményt ad. Ekkor az eredmény sites×trait fajszám.
#
# traittab            A fajok traitjellemzőit tartalmazó táblázat.
#                     Soraiban a fajok, oszlopaiban a trait-csoportok
#                     (pl. feeding group) egyes trait-jellemzői (pl. piscivore).
#                     Az oszlopok a fajok adott traitjellemzőbe való
#                     bináris besorolását kell tartalmazzák
#                     (0,1 adatok), kivéve az első oszlopot.
#                     Az első oszlop a fajnevek cimkéit kell tartalmazza
#                     (character vector!). A fajok sorrendjének, illetve
#                     a neveik cimkéjének meg kell egyezni az x adattábla
#                     oszlopaiban levő fajsorrenddel, illetve fajnévcímkékkel.
#
# method              Az elkészítendő trait adattábla típusára vonatkozó
#                     argumentum.
#                     "specnumb":    adott helyen adott traitjellemzőbe
#                                    tartozó fajok száma.
#                     "relspecnumb": adott helyen adott traitjellemzőbe
#                                    tartozó fajok száma, az adott hely
#                                    összfajszámához viszonyítva.
#                                    Ha a 'nativeonly==TRUE', akkor csak
#                                    a native fajok összfajszámához
#                                    viszonyít. De a 'nonnativespec'
#                                    oszlopban megadja a non-native fajok
#                                    adott helyen belüli relatív fajszámát is.
#                     "abund":       adott helyen adott traitjellemzőbe
#                                    tartozó fajok egyedszáma.
#                     "relabund":    adott helyen adott traitjellemzőbe
#                                    tartozó fajok egyedszáma visznyítva
#                                    az adott hely összegyedszámához.
#                                    Ha a 'nativeonly==TRUE', akkor csak
#                                    a native fajok összegyedszámához
#                                    viszonyít.  De a 'nonnativespec'
#                                    oszlopban megadja a non-native fajok
#                                    adott helyen belüli relatív egyedszámát is.
#                     Lásd alul a többi argumentum leírását!
#
# nativeonly          Logikai. Ha FALSE (default), akkor az x adattáblázatban
#                     szereplő minden fajt felhasznál a trait-adattábla
#                     készítéséhez.
#                     Ha TRUE, akkor a biogegráfiai státusz
#                     trait-csoportot (native vs. non-native) leszámítva
#                     a többi traitcsoportra csak a sites×faj adattáblában
#                     szereplő native fajok adatait használja fel a
#                     trait-adattábla készítéséhez. Ugyanakkor a
#                     biogeográfiai traitcsoportra vonatkozóan megadja:
#                     - a native fajok számát és a non-native fajok számát
#                       (method="specnumb")
#                     - a native fajok számításhoz felhasznált arányát,
#                       (ami 1) és  a non-native fajok helyen belüli
#                       relatív fajszámát (tehát a hely native és nonnative összfajszámán belüli arányukat)
#                       (method="relspecnumb")
#                     - a native fajok egyedszámát és a non-native fajok egyedszámát
#                      (method="abund")
#                     - a native fajok számításhoz felhasznált egyedek arányát,
#                       (ami 1) és  a non-native fajok helyen belüli
#                       relatív egyedszámát (tehát a hely native és nonnative összegyedszámán belüli arányukat)
#                      (method="relabund").
#
# nativespec          A traittab táblázatban annak az oszlopnak a száma
#                     ami a természetesen honos fajokat kódolja.
#                     Akkor szükséges, ha 'nativeonly==TRUE'.
#
# nonnativespec       A traittab táblázatban annak az oszlopnak a száma
#                     ami a non-native fajokat kódolja.
#                     Akkor szükséges, ha 'nativeonly==TRUE'.
#
#
# Fontos:
# Az x adattábla oszlopaiban levő fajsorrendnek, és a fajnevek cimkéinek
# meg kell egyezni a traittab tábla első oszlopában levő fajnévcimkékkel.
#
# Hiányzó adatok a traittab-ban:
# Ha a trait tábla olyan faj tartalmaz, amely valamely traitcsoportra
# nézve nincs besorolva és ezért a faj sorában az adott traitcsoport
# oszlopai NA-kat tartalmaznak, akkor a függvény ezeket az NA-kat
# először 0-ra cseréli, különben a mátrixszorzás nem végezhető el.
#
#
# Sály Péter
# Tihany, 2015.05.06. 15:23:25


## Examples:
##***********
## Trial data:
##-------------
## Sites×species abundance matrix:
#  trialdata <- data.frame("sp1"=c(20, 0, 0,10, 0, 0, 2),
#                          "sp2"=c( 5, 5, 0, 0, 5, 0, 4),
#                          "sp3"=c( 0, 0, 2, 2, 2, 0, 1),
#                          "sp4"=c( 0, 2, 2, 2, 0, 2, 0),
#                          "sp5"=c(10, 0,10, 0, 3, 6, 0))
#  trialdata
#  # A 6-os helyen csak non-native fajok vannak.
#  # A 7-es helyen csak native fajok vannak.
## Species×trait matrix:
##                                     sp1 sp2 sp3 sp4 sp5
#  trialtrait <- data.frame("species"=colnames(trialdata),
#                           "OMN"    =c(1,  0,  0,  1,  1),  # Feeding: omnivore
#                           "PISC"   =c(0,  1,  0,  0,  0),  # Feeding: piscivore
#                           "PLA"    =c(0,  0,  1,  0,  0),  # Feeding: planktivore
#                           "TOL"    =c(1,  0,  0,  0,  1),  # Tolerance: tolerant
#                           "INTOL"  =c(0,  1,  1,  1,  0),  # Tolerance: intolerant
#                           "N"      =c(1,  1,  1,  0,  0),  # Biogeographical status: native
#                           "NN"     =c(0,  0,  0,  1,  1),  # Biogeographical status: non-native
#                           stringsAsFactors=FALSE)
#  trialtrait
#  str(trialtrait)
#  as.matrix(colnames(trialtrait))
#  # nativespec:    column 7
#  # nonnativespec: column 8
#
## Results: Sites×Trait data matrices:
##------------------------------------
## a) All species
#  traitdata(x=trialdata, traittab=trialtrait, method="specnumb",    nativeonly=F)
#  traitdata(x=trialdata, traittab=trialtrait, method="relspecnumb", nativeonly=F)
#  traitdata(x=trialdata, traittab=trialtrait, method="abund",       nativeonly=F)
#  traitdata(x=trialdata, traittab=trialtrait, method="relabund",    nativeonly=F)
#
## b) Only native species
#  traitdata(x=trialdata, traittab=trialtrait, method="specnumb",    nativeonly=T,
#            nonnativespec=8, nativespec=7)
#  traitdata(x=trialdata, traittab=trialtrait, method="relspecnumb", nativeonly=T,
#            nonnativespec=8, nativespec=7)
#  traitdata(x=trialdata, traittab=trialtrait, method="abund",       nativeonly=T,
#            nonnativespec=8, nativespec=7)
#  traitdata(x=trialdata, traittab=trialtrait, method="relabund",    nativeonly=T,
#            nonnativespec=8, nativespec=7)


#---------------------------------------------------------------------
# Function code:
traitdata <- function(x, traittab,
                      method=c("specnumb", "relspecnumb", "abund", "relabund"),
                      nativeonly    = FALSE,
                      nativespec    = NULL,
                      nonnativespec = NULL) {
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Ellenőrzések:
  #**************
  # A fajok sorrendjem egyezik-e az adattábla oszlopaiban és a
  # traittábla soraiban:
  if(!all(traittab[,1] == colnames(x))) {
  stop("\nOrder of the species or the labels of the species' name are not match in 'x' and in 'traittab'.\nPlease check it.\n") }
  
  
  # Ha nativeonly==TRUE, akkor a nativespec és a nonnativespec
  # traitek oszlopszáma be van-e állítva:
  # Ha valamelyik nincs, vagy egyik sem, hibaüzenetet ad.
  if(nativeonly==TRUE & is.null(nativespec)    |
     nativeonly==TRUE & is.null(nonnativespec) |
     nativeonly==TRUE & is.null(nonnativespec) & is.null(nativespec)) {
     stop("\nColumn number of 'native species' and/or 'non-native species' in 'traittab' is not set.\nPlease check it.\n") }
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # A traittab NA-it 0-ra cserélem:
  #********************************
  traittab <- replace(traittab, is.na(traittab),0)
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Method="specnumb":
  #*******************
  if(method=="specnumb") {
    if(nativeonly==FALSE) {
      xdat     <- x
      traitdat <- traittab[,-1]
      
      xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      res
    }
    
    if(nativeonly==TRUE) {
      xdat     <- x[,traittab[,nativespec]==1]            # Csak native fajok.
      traitdat <- traittab[traittab[,nativespec]==1,  -1] # Csak native fajok.
      
      xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      
      # Ebbe a res táblába még a non-native fajok számát be kell írni:
      # Non-nativ oszlopszám a 'res'-ben:
      i <- match(colnames(traittab)[nonnativespec], colnames(res))
      # Beírás:
      res[,i] <- as.matrix(x>0) %*% as.matrix(traittab[,nonnativespec])
      res
      #
      # Mivel nativeonly==TRUE, a native fajok száma trait (N) az
      # adott hely native fajainak számát tartalmazza.
      # Azonban a non-native fajok száma trait (NN), az adott hely
      # non-native fajainak számát tartalmazza (melyek tehát a trait tábla
      # készítéshez nem lettek felhasználva).
    }
  } # End of if(method=="specnumb")
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Method="relspecnumb":
  #**********************
  if(method=="relspecnumb") {
    if(nativeonly==FALSE) {
      xdat     <- x
      traitdat <- traittab[,-1]
      
      xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      totS <- apply(xdat>0,1,sum)
      res  <- sweep(x=res, MARGIN=1, STATS=totS, FUN="/")
      res
    }
    
    if(nativeonly==TRUE) {
      xdat     <- x[,traittab[,nativespec]==1]            # Csak native fajok.
      traitdat <- traittab[traittab[,nativespec]==1,  -1] # Csak native fajok.
      
      xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      totS <- apply(xdat>0,1,sum)
      res  <- sweep(x=res, MARGIN=1, STATS=totS, FUN="/")
      
      # Ha egy helyen nem volt native faj, akkor sz összes trait 0,
      # és az össz native fajszám (totS) is az. A 0/0 osztás miatt az ilyen
      # helyek NaN értéket kapnak. ezeket kicserélem 0-ra:
      res <- replace(res, is.nan(res), 0)
      
      # Ebbe a res táblába még a non-native fajok relatív számát be kell írni:
      # Non-nativ oszlopszám a 'res'-ben:
      i <- match(colnames(traittab)[nonnativespec], colnames(res))
      # A native és non-native fajok együttes összfajszáma:
      totS <- apply(x>0,1,sum)
      # Beírás:
      res[,i] <- as.matrix(x>0) %*% as.matrix(traittab[,nonnativespec])/totS
      res
      #
      # Mivel nativeonly==TRUE, a native fajok aránya trait (N) 1.
      # A non-native fajok aránya trait (NN), azonban az adott hely
      # native ÉS non-native fajokból álló összes fajszámán belül a
      # non-native fajok arányát tartalmazza.
    }
  } # End of if(method=="relspecnumb") {
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Method="abund":
  #****************
  if(method=="abund") {
    if(nativeonly==FALSE) {
      xdat     <- x
      traitdat <- traittab[,-1]
      
      ##xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      res
    }
    
    if(nativeonly==TRUE) {
      xdat     <- x[,traittab[,nativespec]==1]            # Csak native fajok.
      traitdat <- traittab[traittab[,nativespec]==1,  -1] # Csak native fajok.
      
      ##xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      
      # Ebbe a res táblába még a non-native fajok egyedszámát be kell írni:
      # Non-nativ oszlopszám a 'res'-ben:
      i <- match(colnames(traittab)[nonnativespec], colnames(res))
      # Beírás:
      res[,i] <- as.matrix(x) %*% as.matrix(traittab[,nonnativespec])
      res
      #
      # Mivel nativeonly==TRUE, a native fajok egyedszáma trait (N) az
      # adott hely native fajainak egyedszámát tartalmazza.
      # Azonban a non-native fajok egyedszáma trait (NN), az adott hely
      # non-native fajainak egyedszámát tartalmazza (melyek tehát a trait tábla
      # készítéshez nem lettek felhasználva).
    }
  } # End of   if(method=="abund") {
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Method="relabund":
  #*******************
  if(method=="relabund") {
    if(nativeonly==FALSE) {
      xdat     <- x
      traitdat <- traittab[,-1]
      
      ##xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      totN <- apply(xdat,1,sum)
      res  <- sweep(x=res, MARGIN=1, STATS=totN, FUN="/")
      res
    }
    
    if(nativeonly==TRUE) {
      xdat     <- x[,traittab[,nativespec]==1]            # Csak native fajok.
      traitdat <- traittab[traittab[,nativespec]==1,  -1] # Csak native fajok.
      
      ##xdat <- replace(xdat, xdat>0,1)
      res  <- as.matrix(xdat) %*% as.matrix(traitdat)
      totN <- apply(xdat,1,sum)
      res  <- sweep(x=res, MARGIN=1, STATS=totN, FUN="/")
      
      # Ha egy helyen nem volt native faj, akkor sz összes trait 0,
      # és az össz native egyedszám (totN) is az. A 0/0 osztás miatt az ilyen
      # helyek NaN értéket kapnak. ezeket kicserélem 0-ra:
      res <- replace(res, is.nan(res), 0)
      
      # Ebbe a res táblába még a non-native fajok relatív számát be kell írni:
      # Non-nativ oszlopszám a 'res'-ben:
      i <- match(colnames(traittab)[nonnativespec], colnames(res))
      # A native és non-native fajok együttes összfajszáma:
      totN <- apply(x,1,sum)
      # Beírás:
      res[,i] <- as.matrix(x) %*% as.matrix(traittab[,nonnativespec])/totN
      res
      #
      # Mivel nativeonly==TRUE, a native fajok relativ abundanciája 
      # trait (N) 1. A non-native fajok relativ abundanciája trait (NN),
      # azonban az adott hely native ÉS non-native fajokból álló
      # összes egyedszámán belül a non-native fajok egyedszámának
      # arányát tartalmazza.
    }
  } # End of   if(method=="relabund") {
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Figyelmeztetés ha nativeonly=TRUE:
  #***********************************
  if(nativeonly==TRUE){
    warning("Trait table was made only from native species.")
  }
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Eredményobjektum:
  #******************
  #str(res)
  res     <- as.data.frame(res)
  res
  #str(res)
  return(res)
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
} # *** End of function ***
#=====================================================================


